
/*
 * sentryc.c - sentry network client
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

#include <signal.h>
#include <syslog.h>
#include <errno.h>

#include "sentry.h"
#include "sentryc.h"

int debug;
int verbose;

char    sentry_server[128];
int     ctrlfd = -1;
int     datafd = -1;
int     data_port = 0;

char    *lockfile = "/var/tmp/sentryc.pid";

int     new_udp_socket(int port);
void    init_sentryc();
void    usage();


void complete()
{
  syslog(LOG_NOTICE, "sentryc closed.\n");
  closelog();
  unlink(lockfile);
  exit (0);
}


int new_udp_socket(int port)
{
  int sockfd;
  int flag=1;

  if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    fprintf(stderr, "sentryc: cannot make new UDP socket.: %s\n", sys_errlist[errno]);
    return -1;
  }
  setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&flag, sizeof(flag));

  return sockfd;
}

int build_ctrl_port(int port) 
{
  int sockfd;
  struct sockaddr_in server;
  struct hostent *hp;

  sockfd = new_udp_socket(port);

  bzero((char *)&server, sizeof(server));
  server.sin_family = AF_INET;
  server.sin_port = htons(server_port);
  if ((server.sin_addr.s_addr = inet_addr(sentry_server)) == INADDR_NONE) {
    hp = gethostbyname(sentry_server);
    if (hp->h_addrtype != AF_INET) {
      fprintf(stderr, "%s: invalid address type.\n", sentry_server);
      close(sockfd);
      return -1;
    }
    bcopy(hp->h_addr, (char *)&server.sin_addr.s_addr, hp->h_length);
  }

  if (connect(sockfd, (struct sockaddr *)&server, sizeof(server)) < 0) {
    fprintf(stderr, "cannot bind or connect: %s\n", sys_errlist[errno]);
    close(sockfd);
    return -1;
  }
  fprintf(stderr, "prepared ctrl socket# %d\n", sockfd);

  return sockfd;
}

int build_data_port(int port)
{
  int sockfd;
  struct sockaddr_in me;
  int namelen;
  
  sockfd = new_udp_socket(port);
  
  bzero((char *)&me, sizeof(me));
  me.sin_family = AF_INET;
  me.sin_addr.s_addr = htonl(INADDR_ANY);
  me.sin_port = 0;

  if (bind(sockfd, (struct sockaddr *)&me, sizeof(me)) < 0) {
    fprintf(stderr, "Cannot bind data socket: %s\n", sys_errlist[errno]);
    close(sockfd);
    return -1;
  }

  /* get data port# */
  if (getsockname(sockfd, (struct sockaddr *)&me, &namelen) < 0) {
    fprintf(stderr, "Cannot get data sockname: %s\n", sys_errlist[errno]);
    close(sockfd);
    return -1;
  }
  data_port = ntohs(me.sin_port);

  fprintf(stderr, "prepared data port# %d, ctrlfd# %d.\n", data_port, sockfd);

  return sockfd;
}


void
mainloop(void)
{
  extern u_char client_type;

  init_ibuf();

  init_request();
  request_call(client_type);

  while (1) {
    get_data(datafd);
  }
}

int
main(int argc, char **argv)
{
  int c;
  extern char *optarg;
  extern int optind;

  debug = 1;                            /* debug flag */
  verbose = 1;

  openlog("sentryc", LOG_PID, LOG_DAEMON);
  syslog(LOG_NOTICE, "sentryc starts.\n");

  signal(SIGINT, complete);
  signal(SIGTERM, complete);
  signal(SIGINT, complete);

  while ((c = getopt(argc, argv, "hdv:p:s:t:L:")) != -1) {
    switch(c) {
    case 'h': /* help */
    default:
      usage();
      exit (1);
    case 'd':
      debug ++;
      break;
    case 'v': /* verbose mode */
      verbose = 1;
      break;
    case 'p':
      server_port = atoi(optarg);
      break;
    case 's':
      shm_size = atoi(optarg);
      break;
	case 't':
#if 0
	  /* toggle */
	  if(client_type == OPEN_UDP_UNICAST)
		  client_type = OPEN_UDP_MULTICAST;
	  else
		  client_type = OPEN_UDP_UNICAST;
#endif
	  if(atoi(optarg)==1) 
		  client_type = OPEN_UDP_MULTICAST;
	  else
		  client_type = OPEN_UDP_UNICAST;
	  
	  break;
	case 'L':
	  lockfile = strdup(optarg);
	  break;
    }      
  }

  if (argc <= 1) 
    usage();

  /*
   * daemonize if debug == 0
   */
  if (debug == 0) { /* future work */ }

  strcpy(sentry_server, argv[argc-1]);

  init_sentryc();

  ctrlfd = build_ctrl_port(0);
  datafd = build_data_port(0);
  if (ctrlfd < 0 || datafd < 0) 
    exit (1);

  fprintf(stderr, "ctrlfd# %d, datafd# %d\n", ctrlfd, datafd);
  fprintf(stderr, " data_port# %d\n", data_port);

  (void)mainloop();
}


void
init_sentryc()
{
  int chk;
  FILE *fp;
  char mesg[80];

  if (debug)
    fprintf(stderr, "inti_sentryc: lockfile=%s\n", lockfile);

  while (1) {
    chk = access(lockfile, F_OK);
    if (chk < 0) {
		if (errno == EINTR) {
			continue;
		}
		if(errno == ENOENT) {
			break;
		}
	}
    fprintf(stderr, "init_sentryc: lockfile(%s) is found.: %s\n", 
	    lockfile, strerror(errno));
    exit (1);
  }

  if ((fp = fopen(lockfile, "w")) == NULL) {
    fprintf(stderr, "init_sentryc: cannot open lockfile(%s): %s\n",
	    lockfile, strerror(errno));
    exit (1);
  }

  sprintf(mesg, "%d\n", getpid());
  fprintf(stderr, "pid# %d\n", getpid());
  
  if (fputs(mesg, fp) == EOF) {
    fprintf(stderr, "init_sentryc: cannot fputs(): %s.\n", sys_errlist[errno]);
    exit (1);
  }
  fclose(fp);

  unlink(lockfile);
}
  

void
usage(void)
{
  fprintf(stderr, "Usage: sentryc [options] hostname.\n\n");
  fprintf(stderr, 
"option:\n\
        -h         help\n\
        -t type    unicast(0)/multicast(1) [default: unicast(0)]\n\
        -c color   monochrome(0)/color(1)  [default: color(1)]\n\
        -s type    frame size\n\
        -r num     frame rate (fps)        [default: 10 fps]\n\
\n");
  exit (1);
}
